==> atopile-0.2.49.tar.gz: atopile-0.2.49/src/atopile/cli/build.py <==
"""CLI command definition for `ato build`."""
import itertools
import json
import logging
import shutil
from functools import wraps
from typing import Callable, Optional

import click

import atopile.assertions
import atopile.bom
import atopile.config
import atopile.front_end
import atopile.layout
import atopile.manufacturing_data
import atopile.netlist
import atopile.variable_report
from atopile.cli.common import project_options
from atopile.components import download_footprint
from atopile.config import BuildContext
from atopile.errors import handle_ato_errors, iter_through_errors
from atopile.instance_methods import all_descendants, match_components
from atopile.netlist import get_netlist_as_str

log = logging.getLogger(__name__)


@click.command()
@project_options
def build(build_ctxs: list[BuildContext]):
    """
    Build the specified --target(s) or the targets specified by the build config.
    Specify the root source file with the argument SOURCE.
    eg. `ato build --target my_target path/to/source.ato:module.path`
    """
    for err_cltr, build_ctx in iter_through_errors(build_ctxs):
        log.info("Building %s", build_ctx.name)
        with err_cltr():
            _do_build(build_ctx)

    project_context = atopile.config.get_project_context()

    # FIXME: this should be done elsewhere, but there's no other "overview"
    # that can see all the builds simultaneously
    manifest = {}
    manifest["version"] = "2.0"
    for ctx in build_ctxs:
        if ctx.layout_path:
            by_layout_manifest = manifest.setdefault("by-layout", {}).setdefault(str(ctx.layout_path), {})
            by_layout_manifest["layouts"] = str(ctx.output_base.with_suffix(".layouts.json"))

    manifest_path = project_context.project_path / "build" / "manifest.json"
    manifest_path.parent.mkdir(exist_ok=True, parents=True)
    with open(manifest_path, "w", encoding="utf-8") as f:
        json.dump(manifest, f)


def _do_build(build_ctx: BuildContext) -> None:
    """Execute a specific build."""
    # Solve the unknown variables
    atopile.assertions.simplify_expressions(build_ctx.entry)
    atopile.assertions.solve_assertions(build_ctx)
    atopile.assertions.simplify_expressions(build_ctx.entry)

    # Ensure the build directory exists
    log.info("Writing outputs to %s", build_ctx.build_path)
    build_ctx.build_path.mkdir(parents=True, exist_ok=True)

    if build_ctx.targets == ["__default__"]:
        targets = muster.do_by_default
    elif build_ctx.targets == ["*"] or build_ctx.targets == ["all"]:
        targets = list(muster.targets.keys())
    else:
        targets = build_ctx.targets

    build_ctx.output_base.parent.mkdir(parents=True, exist_ok=True)

    built_targets = []
    for err_cltr, target_name in iter_through_errors(targets):
        with err_cltr():
            muster.targets[target_name](build_ctx)
        built_targets.append(target_name)

    log.info(f"Successfully built '{', '.join(built_targets)}' for '{build_ctx.name}' config")


TargetType = Callable[[BuildContext], None]


class Muster:
    """A class to register targets to."""

    def __init__(self, logger: Optional[logging.Logger] = None) -> None:
        self.targets = {}
        self.do_by_default = []
        self.log = logger or logging.getLogger(__name__)

    def add_target(self, func: TargetType, name: Optional[str] = None, default: bool = True):
        """Register a function as a target."""
        name = name or func.__name__
        self.targets[name] = func
        if default:
            self.do_by_default.append(name)
        return func

    def register(self, name: Optional[str] = None, default: bool = True):
        """Register a target under a given name."""

        def decorator(func: TargetType):
            @wraps(func)
            def wrapper(build_args: BuildContext):
                with handle_ato_errors():
                    return func(build_args)

            self.add_target(wrapper, name, default)
            return wrapper

        return decorator


muster = Muster()


@muster.register("copy-footprints")
def consolidate_footprints(build_args: BuildContext) -> None:
    """Consolidate all the project's footprints into a single directory."""
    fp_target = build_args.build_path / "footprints" / "footprints.pretty"
    fp_target_step = build_args.build_path / "footprints" / "footprints.3dshapes"
    fp_target.mkdir(exist_ok=True, parents=True)

    for fp in atopile.config.get_project_context().project_path.glob("**/*.kicad_mod"):
        try:
            shutil.copy(fp, fp_target)
        except shutil.SameFileError:
            log.debug("Footprint %s already exists in the target directory", fp)

    # Post-process all the footprints in the target directory
    for fp in fp_target.glob("**/*.kicad_mod"):
        with open(fp, "r+", encoding="utf-8") as file:
            content = file.read()
            content = content.replace("{build_dir}", str(fp_target_step))
            file.seek(0)
            file.write(content)
            file.truncate()


@muster.register("copy-3dmodels")
def consolidate_3dmodels(build_args: BuildContext) -> None:
    """Consolidate all the project's 3d models into a single directory."""
    fp_target = build_args.build_path / "footprints" / "footprints.3dshapes"
    fp_target.mkdir(exist_ok=True, parents=True)

    prj_path = atopile.config.get_project_context().project_path

    for fp in itertools.chain(prj_path.glob("**/*.step"), prj_path.glob("**/*.wrl")):
        try:
            shutil.copy(fp, fp_target)
        except shutil.SameFileError:
            log.debug("Footprint %s already exists in the target directory", fp)


@muster.register("netlist")
def generate_netlist(build_args: BuildContext) -> None:
    """Generate a netlist for the project."""
    with open(build_args.output_base.with_suffix(".net"), "w", encoding="utf-8") as f:
        f.write(get_netlist_as_str(build_args.entry))


@muster.register("bom")
def generate_bom(build_args: BuildContext) -> None:
    """Generate a BOM for the project."""
    with open(build_args.output_base.with_suffix(".csv"), "w", encoding="utf-8") as f:
        f.write(atopile.bom.generate_bom(build_args.entry))


@muster.register("designator-map")
def generate_designator_map(build_args: BuildContext) -> None:
    """Generate a designator map for the project."""
    atopile.bom.generate_designator_map(build_args.entry)


@muster.register("mfg-data", default=False)
def generate_manufacturing_data(build_ctx: BuildContext) -> None:
    """Generate a designator map for the project."""
    atopile.manufacturing_data.generate_manufacturing_data(build_ctx)


@muster.register("drc", default=False)
def generate_drc_report(build_ctx: BuildContext) -> None:
    """Generate a designator map for the project."""
    atopile.manufacturing_data.generate_drc_report(build_ctx)


@muster.register("clone-footprints")
def clone_footprints(build_args: BuildContext) -> None:
    """Clone the footprints for the project."""
    all_components = filter(match_components, all_descendants(build_args.entry))

    for component in all_components:
        log.debug("Cloning footprint for %s", component)
        download_footprint(component, footprint_dir=build_args.build_path / "footprints/footprints.pretty")


@muster.register("layout-module-map")
def generate_module_map(build_args: BuildContext) -> None:
    """Generate a designator map for the project."""
    atopile.layout.generate_module_map(build_args)


@muster.register("assertions-report")
def generate_assertion_report(build_ctx: BuildContext) -> None:
    """Generate a report based on assertions made in the source code."""
    atopile.assertions.generate_assertion_report(build_ctx)

==> atopile-0.3.0.dev0.tar.gz: atopile-0.3.0.dev0/src/atopile/cli/build.py <==
"""CLI command definition for `ato build`."""

import logging
import shutil
from functools import wraps
from typing import Callable, Optional

import click

import atopile.bom
import atopile.front_end
import atopile.netlist
import atopile.manufacturing_data
from atopile.cli.common import project_options
from atopile.config import BuildContext
from atopile.errors import (
    handle_ato_errors,
    iter_through_errors,
    muffle_fatalities,
)
from atopile.front_end import set_search_paths
from atopile.netlist import get_netlist_as_str
from atopile.instance_methods import all_descendants, match_components
from atopile.components import download_footprint, configure_cache

log = logging.getLogger(__name__)

@click.command()
@project_options
@muffle_fatalities
def build(build_ctxs: list[BuildContext]):
    """
    Build the specified --target(s) or the targets specified by the build config.
    Specify the root source file with the argument SOURCE.
    eg. `ato build --target my_target path/to/source.ato:module.path`
    """
    for err_cltr, build_ctx in iter_through_errors(build_ctxs):
        log.info("Building %s", build_ctx.name)
        with err_cltr():
            _do_build(build_ctx)


def _do_build(build_ctx: BuildContext) -> None:
    """Execute a specific build."""
    # Set the search paths for the front end
    set_search_paths([build_ctx.src_path, build_ctx.module_path])

    # Configure the cache for component data
    configure_cache(build_ctx.project_path)

    # Ensure the build directory exists
    log.info("Writing outputs to %s", build_ctx.build_path)
    build_ctx.build_path.mkdir(parents=True, exist_ok=True)

    if build_ctx.targets == ["__default__"]:
        targets = muster.do_by_default
    elif build_ctx.targets == ["*"] or build_ctx.targets == ["all"]:
        targets = list(muster.targets.keys())
    else:
        targets = build_ctx.targets

    for err_cltr, target_name in iter_through_errors(targets):
        log.info("Building %s", target_name)
        with err_cltr():
            muster.targets[target_name](build_ctx)


TargetType = Callable[[BuildContext], None]


class Muster:
    """A class to register targets to."""

    def __init__(self, logger: Optional[logging.Logger] = None) -> None:
        self.targets = {}
        self.do_by_default = []
        self.log = logger or logging.getLogger(__name__)

    def add_target(self, func: TargetType, name: Optional[str] = None, default: bool = True):
        """Register a function as a target."""
        name = name or func.__name__
        self.targets[name] = func
        if default:
            self.do_by_default.append(name)
        return func

    def register(self, name: Optional[str] = None, default: bool = True):
        """Register a target under a given name."""

        def decorator(func: TargetType):
            @wraps(func)
            def wrapper(build_args: BuildContext):
                with handle_ato_errors():
                    return func(build_args)

            self.add_target(wrapper, name, default)
            return wrapper

        return decorator


muster = Muster()


@muster.register("copy-footprints")
def consolidate_footprints(build_args: BuildContext) -> None:
    """Consolidate all the project's footprints into a single directory."""
    fp_target = build_args.build_path / "footprints" / "footprints.pretty"
    fp_target.mkdir(exist_ok=True, parents=True)

    for fp in build_args.project_path.glob("**/*.kicad_mod"):
        try:
            shutil.copy(fp, fp_target)
        except shutil.SameFileError:
            log.debug("Footprint %s already exists in the target directory", fp)


@muster.register("netlist")
def generate_netlist(build_args: BuildContext) -> None:
    """Generate a netlist for the project."""
    with open(build_args.output_base.with_suffix(".net"), "w", encoding="utf-8") as f:
        f.write(get_netlist_as_str(build_args.entry))


@muster.register("bom")
def generate_bom(build_args: BuildContext) -> None:
    """Generate a BOM for the project."""
    with open(build_args.output_base.with_suffix(".csv"), "w", encoding="utf-8") as f:
        f.write(atopile.bom.generate_bom(build_args.entry))


@muster.register("designator-map")
def generate_designator_map(build_args: BuildContext) -> None:
    """Generate a designator map for the project."""
    atopile.bom.generate_designator_map(build_args.entry)


@muster.register("mfg-data", default=False)
def generate_manufacturing_data(build_ctx: BuildContext) -> None:
    """Generate a designator map for the project."""
    atopile.manufacturing_data.generate_manufacturing_data(build_ctx)

@muster.register("clone-footprints")
def clone_footprints(build_args: BuildContext) -> None:
    """Clone the footprints for the project."""
    all_components = filter(match_components, all_descendants(build_args.entry))

    for component in all_components:
        log.debug("Cloning footprint for %s", component)
        download_footprint(component, footprint_dir=build_args.build_path / "build/footprints/footprints.pretty")
